# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2024/7/29 14:25
# Author     ：Maxwell
# Description：
"""
import aiohttp
from hwhkit.utils.http.HttpResult import HttpResult


class HttpAsyncClient:

    @staticmethod
    async def post(url, headers=None, data=None, json=None, files=None) -> HttpResult:
        if data:
            form_data = aiohttp.FormData()
            for key, value in data.items():
                form_data.add_field(key, value)

            if files:
                for key, file in files.items():
                    form_data.add_field(key, file, filename='file', content_type='application/octet-stream')

            async with aiohttp.ClientSession() as session:
                async with session.post(url, headers=headers, data=form_data) as response:
                    succeed = response.status == 200
                    content = await response.read()
                    text = await response.text()
                    message = f'{response.status}:{response.reason}'
                    return HttpResult(succeed=succeed, content=content, text=text, message=message)

        if json:
            async with aiohttp.ClientSession() as session:
                async with session.post(url, headers=headers, json=json) as response:
                    succeed = response.status == 200
                    content = await response.read()
                    text = await response.text()
                    message = f'{response.status}:{response.reason}'
                    return HttpResult(succeed=succeed, content=content, text=text, message=message)

    @staticmethod
    async def get_content(url) -> HttpResult:
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                succeed = response.status == 200
                content = await response.read()
                message = f'{response.status}:{response.reason}'
                return HttpResult(succeed=succeed, content=content, message=message)





# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/1/4 11:05
# Author     ：Maxwell
# Description：
"""


from abc import ABC, abstractmethod


class PluginBase(ABC):
    @abstractmethod
    def on_message_published(self, topic: str, message: str) -> str:
        pass

    @abstractmethod
    def on_message_received(self, topic: str, message: str) -> str:
        pass



import yaml
from typing import Type

from hwhkit.llm.models import LanguageModelFactory, LanguageModelProperty, LanguageModel
from hwhkit.llm.models.alibaba import AlibabaStrategy
from hwhkit.llm.models.anthropic import AnthropicStrategy
from hwhkit.llm.models.deepseek import DeepSeekStrategy
from hwhkit.llm.models.lingyi import LingYiStrategy
from hwhkit.llm.models.moonshot import MoonshotStrategy
from hwhkit.llm.models.nvidia import NvidiaStrategy
from hwhkit.llm.models.openai import OpenaiStrategy
from hwhkit.llm.models.siliconflow import SiliconFlowStrategy
from hwhkit.llm.models.zhipu import ZhipuStrategy

provider_map = {
    "alibaba": AlibabaStrategy,
    "anthropic": AnthropicStrategy,
    "deepseek": DeepSeekStrategy,
    "lingyi": LingYiStrategy,
    "moonshot": MoonshotStrategy,
    "nvidia": NvidiaStrategy,
    "openai": OpenaiStrategy,
    "siliconflow": SiliconFlowStrategy,
    "zhipu": ZhipuStrategy,
}


def load_models_from_yaml(
        config_file: str = 'llm_config.yaml', keys_file: str = "llm_key.yaml"
) -> Type[LanguageModelFactory]:
    with open(config_file, "r") as f:
        config = yaml.safe_load(f)

    with open(keys_file, "r") as f:
        keys = yaml.safe_load(f).get("keys", {})

    models_config = config.get("models", {})
    for provider, provider_config in models_config.items():
        for model_name, model_config in provider_config.items():
            model_keys = {}
            for key_config in model_config.get("keys", []):
                key_name = key_config["name"]
                if key_name in keys:
                    model_keys[key_name] = keys[key_name]

            provider_strategy = provider_map.get(provider)
            if provider_strategy:
                model = LanguageModel(provider_strategy(model_keys, LanguageModelProperty(**model_config)))
                LanguageModelFactory.register_model_instance(model_name, model)

    return LanguageModelFactory



import os
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography.hazmat.primitives import serialization, hashes
from cryptography.hazmat.primitives.kdf.hkdf import HKDF
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import padding
from cryptography.exceptions import InvalidKey, UnsupportedAlgorithm


class ECCCipher:
    CURVE = ec.SECP521R1()
    HASH_ALGORITHM = hashes.SHA512()
    HASH_LENGTH = 64
    DERIVED_KEY_LENGTH = 64
    AES_KEY_LENGTH = 32
    IV_LENGTH = 16
    INFO = b"ecc-encryption"

    def __init__(self, private_key=None, public_key=None):
        if private_key and public_key:
            self.private_key = private_key
            self.public_key = public_key
        else:
            self.private_key, self.public_key = self.generate_key_pair()

    def generate_key_pair(self):
        private_key = ec.generate_private_key(self.CURVE)
        public_key = private_key.public_key()
        return private_key, public_key

    def encrypt(self, plaintext: bytes, peer_public_key) -> bytes:
        try:
            shared_key = self.private_key.exchange(ec.ECDH(), peer_public_key)
            derived_key = HKDF(
                algorithm=self.HASH_ALGORITHM,
                length=self.DERIVED_KEY_LENGTH,
                salt=None,
                info=self.INFO,
            ).derive(shared_key)
            iv = os.urandom(self.IV_LENGTH)
            cipher = Cipher(algorithms.AES(derived_key[:self.AES_KEY_LENGTH]), modes.CBC(iv))
            encryptor = cipher.encryptor()
            padder = padding.PKCS7(algorithms.AES.block_size).padder()
            padded_data = padder.update(plaintext) + padder.finalize()
            ciphertext = encryptor.update(padded_data) + encryptor.finalize()
            return iv + ciphertext
        except ValueError as e:
            raise ValueError(f"Encryption failed: {str(e)}")
        except InvalidKey:
            raise ValueError("Encryption failed: Invalid public key.")
        except UnsupportedAlgorithm:
            raise ValueError("Encryption failed: Unsupported algorithm.")
        except Exception as e:
            raise ValueError(f"Encryption failed: {str(e)}")

    def decrypt(self, ciphertext: bytes, peer_public_key) -> bytes:
        try:
            iv = ciphertext[:self.IV_LENGTH]
            ciphertext = ciphertext[self.IV_LENGTH:]
            shared_key = self.private_key.exchange(ec.ECDH(), peer_public_key)
            derived_key = HKDF(
                algorithm=self.HASH_ALGORITHM,
                length=self.DERIVED_KEY_LENGTH,
                salt=None,
                info=self.INFO,
            ).derive(shared_key)
            cipher = Cipher(algorithms.AES(derived_key[:self.AES_KEY_LENGTH]), modes.CBC(iv))
            decryptor = cipher.decryptor()
            padded_data = decryptor.update(ciphertext) + decryptor.finalize()
            unpadder = padding.PKCS7(algorithms.AES.block_size).unpadder()
            plaintext = unpadder.update(padded_data) + unpadder.finalize()
            return plaintext
        except ValueError as e:
            raise ValueError(f"Decryption failed: {str(e)}")
        except InvalidKey:
            raise ValueError("Decryption failed: Invalid private key.")
        except UnsupportedAlgorithm:
            raise ValueError("Decryption failed: Unsupported algorithm.")
        except Exception as e:
            raise ValueError(f"Decryption failed: {str(e)}")

    def serialize_private_key(self) -> bytes:
        return self.private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )

    def serialize_public_key(self) -> bytes:
        return self.public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )

    @staticmethod
    def deserialize_private_key(private_key_pem: bytes):
        return serialization.load_pem_private_key(
            private_key_pem,
            password=None,
        )

    @staticmethod
    def deserialize_public_key(public_key_pem: bytes):
        return serialization.load_pem_public_key(public_key_pem)


if __name__ == '__main__':
    alice_cipher = ECCCipher()
    bob_cipher = ECCCipher()

    plaintext = b"Hello, ECC encryption asdfasdfasdfasdfasdfaasdfsadfasdfasdfasdfasdf!"
    ciphertext = alice_cipher.encrypt(plaintext, bob_cipher.public_key)
    print("Ciphertext:", ciphertext.hex())

    decrypted_text = bob_cipher.decrypt(ciphertext, alice_cipher.public_key)
    print("Decrypted text:", decrypted_text.decode('utf-8'))


import traceback
import random
from openai import OpenAI
from typing import List, Dict, Optional, AsyncGenerator
from hwhkit.utils import logger
from hwhkit.llm.models import ModelStrategy, LanguageModelProperty


class DeepSeekStrategy(ModelStrategy):
    def __init__(self, keys: Dict[str, str], model_property: LanguageModelProperty):
        self._keys = keys
        self._property = model_property
        self._client_pool = self._initialize_client_pool()

    def get_name(self) -> str:
        return "deepseek"

    def base_url(self) -> str:
        return "https://api.deepseek.com/v1"

    def _initialize_client_pool(self) -> List[Dict[str, OpenAI]]:
        return [{"name": k, "client": OpenAI(api_key=v, base_url=self.base_url())} for k, v in self._keys.items()]

    def _get_client(self) -> Dict[str, OpenAI]:
        if not self._client_pool:
            raise Exception("No DeepSeek client available.")
        return random.choice(self._client_pool)

    async def gen_messages(
            self, question: str, system_prompt: Optional[str] = None, history_messages: Optional[List[Dict]] = None
    ) -> List[Dict[str, str]]:
        messages = []
        if system_prompt:
            messages.append({'role': 'system', 'content': system_prompt})
        if history_messages:
            messages.extend(history_messages)
        if question:
            messages.append({'role': 'user', 'content': question})
        return messages

    async def chat(self, messages: List[Dict[str, str]]) -> str:
        client_info = self._get_client()
        client_name, client = client_info["name"], client_info["client"]

        try:
            response = client.chat.completions.create(
                model=self._property.name,
                messages=messages,
                timeout=self._property.timeout,
                temperature=self._property.temperature,
                top_p=self._property.top_p,
                max_tokens=self._property.max_output_token
            )
            return response.choices[0].message.content
        except Exception as e:
            logger.error(
                f"\nContent Generate error: "
                f"\nLLM {self._property.name}@key: {client_name}"
                f"\nError: {traceback.format_exc()}"
            )
            raise e

    async def chat_stream(self, messages: List[Dict[str, str]]) -> AsyncGenerator[str, None]:
        client_info = self._get_client()
        client_name, client = client_info["name"], client_info["client"]

        try:
            for part in client.chat.completions.create(
                model=self._property.name,
                messages=messages,
                timeout=self._property.timeout,
                temperature=self._property.temperature,
                top_p=self._property.top_p,
                max_tokens=self._property.max_output_token,
                stream=True
            ):
                if part.choices:
                    content = part.choices[0].delta.content
                    if content:
                        yield content
        except Exception as e:
            logger.error(
                f"\nContent Generate Stream error: "
                f"\nLLM {self._property.name}@key: {client_name}"
                f"\nError: {traceback.format_exc()}"
            )
            raise e



# !/usr/bin/env python
# -*-coding:utf-8 -*-

"""
# Time       ：2025/1/4 11:05
# Author     ：Maxwell
# Description：
"""

from hwhkit.connection.mqtt.plugins import proto_pb2
from hwhkit.connection.mqtt.plugins import PluginBase


class ProtobufPlugin(PluginBase):
    def on_message_published(self, topic: str, message: str) -> str:
        payload = proto_pb2.EncryptedPayload(data=message.encode('utf-8'))
        return payload.SerializeToString()

    def on_message_received(self, topic: str, message: str) -> str:
        payload_proto = proto_pb2.EncryptedPayload()
        payload_proto.ParseFromString(message)
        return payload_proto.data.decode('utf-8')


import base64
import yaml
import hashlib
import struct
import zlib
from pathlib import Path
from typing import Dict
from hwhkit.utils.security.rsa import RSACipher
from hwhkit.utils.security.ecc import ECCCipher
from hwhkit.utils.security.aes import AESCipher
from hwhkit.connection.mqtt.plugins import PluginBase


class EncryptionPlugin(PluginBase):
    def __init__(self, key_pairs_file: str):
        self.topic_keys: Dict[str, Dict[str, str]] = {}
        self.load_key_pairs(key_pairs_file)

    def load_key_pairs(self, key_pairs_file: str):
        if not Path(key_pairs_file).exists():
            self.generate_default_yaml(key_pairs_file)
        with open(key_pairs_file, "r") as f:
            data = yaml.safe_load(f)
        if "key_pairs" not in data:
            raise ValueError("Invalid key_pairs file format: missing 'key_pairs' key")
        self.topic_keys = data["key_pairs"]

    def generate_default_yaml(self, key_pairs_file: str):
        rsa = RSACipher()
        default_data = {
            "key_pairs": {
                "default_topic": {
                    "algorithm": "rsa",
                    "public": rsa.serialize_public_key().decode('utf-8').strip(),
                    "private": rsa.serialize_private_key().decode('utf-8').strip()
                }
            }
        }
        with open(key_pairs_file, "w") as f:
            yaml.safe_dump(default_data, f)
        print(f"Generated default YAML file at: {key_pairs_file}")

    def get_keys(self, topic: str) -> Dict[str, str]:
        if topic not in self.topic_keys:
            raise ValueError(f"No keypair found for topic: {topic}")
        return self.topic_keys[topic]

    def hash_key(self, key: str) -> bytes:
        hasher = hashlib.sha256()
        hasher.update(key.encode('utf-8'))
        return hasher.digest()

    def encode_payload(self, algorithm: str, encrypted_aes_key: bytes, encrypted_message: bytes) -> str:
        if algorithm == "aes":
            algorithm_code = 0
        elif algorithm == "rsa":
            algorithm_code = 1
        elif algorithm == "ecc":
            algorithm_code = 2
        else:
            raise ValueError(f"Unsupported algorithm: {algorithm}")
        if len(encrypted_message) > 50000:
            raise ValueError("Message length exceeds 50,000 characters")
        payload = bytearray()
        payload.append(algorithm_code)
        payload.extend(struct.pack('>H', len(encrypted_message)))
        payload.extend(encrypted_message)
        if algorithm != "aes":
            payload.extend(encrypted_aes_key)
        crc32 = zlib.crc32(payload)
        payload.extend(struct.pack('>I', crc32))
        return base64.urlsafe_b64encode(payload).decode('utf-8')

    def decode_payload(self, payload: str) -> tuple:
        payload_bytes = base64.urlsafe_b64decode(payload + '=' * (-len(payload) % 4))
        crc32 = struct.unpack('>I', payload_bytes[-4:])[0]
        if zlib.crc32(payload_bytes[:-4]) != crc32:
            raise ValueError("Payload checksum mismatch")
        algorithm_code = payload_bytes[0]
        if algorithm_code == 0:
            algorithm = "aes"
        elif algorithm_code == 1:
            algorithm = "rsa"
        elif algorithm_code == 2:
            algorithm = "ecc"
        else:
            raise ValueError("Invalid algorithm code in payload")
        v_length = struct.unpack('>H', payload_bytes[1:3])[0]
        if v_length > 50000:
            raise ValueError("Message length exceeds 50,000 characters")
        v_start = 3
        v_end = v_start + v_length
        v = payload_bytes[v_start:v_end]
        k = payload_bytes[v_end:-4] if algorithm != "aes" else None
        return algorithm, k, v

    def on_message_published(self, topic: str, message: str) -> str:
        try:
            keys = self.get_keys(topic)
            algorithm = keys.get("algorithm", "rsa")
            if algorithm == "aes":
                aes_key = keys.get("private")
                if not aes_key:
                    raise ValueError("AES key is missing for AES algorithm")
                hashed_key = self.hash_key(aes_key)
                aes_cipher = AESCipher(key=hashed_key)
                encrypted_message = aes_cipher.encrypt(message).encode('utf-8')
                encrypted_aes_key = b""
            else:
                aes_cipher = AESCipher()
                aes_key = aes_cipher.get_key()
                if algorithm == "rsa":
                    public_key_pem = keys["public"]
                    public_key_pem_bytes = public_key_pem.encode('utf-8')
                    public_key = RSACipher.deserialize_public_key(public_key_pem_bytes)
                    cipher = RSACipher(public_key=public_key)
                elif algorithm == "ecc":
                    public_key_pem = keys["public"]
                    public_key_pem_bytes = public_key_pem.encode('utf-8')
                    public_key = ECCCipher.deserialize_public_key(public_key_pem_bytes)
                    cipher = ECCCipher(public_key=public_key)
                else:
                    raise ValueError(f"Unsupported algorithm: {algorithm}")
                encrypted_aes_key = cipher.encrypt(aes_key.encode('utf-8'))
                encrypted_message = aes_cipher.encrypt(message).encode('utf-8')
            payload = self.encode_payload(algorithm, encrypted_aes_key, encrypted_message)
            return payload
        except Exception as e:
            raise ValueError(f"Encryption failed: {str(e)}")

    def on_message_received(self, topic: str, message: bytes) -> str:
        try:
            keys = self.get_keys(topic)
            algorithm, k, v = self.decode_payload(message.decode('utf-8'))
            if algorithm == "aes":
                aes_key = keys.get("private")
                if not aes_key:
                    raise ValueError("AES key is missing for AES algorithm")
                hashed_key = self.hash_key(aes_key)
                aes_cipher = AESCipher(key=hashed_key)
                return aes_cipher.decrypt(v.decode('utf-8'))
            else:
                if k is None:
                    raise ValueError("Invalid payload format: missing 'k' for RSA/ECC")
                private_key_pem = keys["private"]
                private_key_pem_bytes = private_key_pem.encode('utf-8')
                if algorithm == "rsa":
                    private_key = RSACipher.deserialize_private_key(private_key_pem_bytes)
                    cipher = RSACipher(private_key=private_key)
                elif algorithm == "ecc":
                    private_key = ECCCipher.deserialize_private_key(private_key_pem_bytes)
                    cipher = ECCCipher(private_key=private_key)
                else:
                    raise ValueError(f"Unsupported algorithm: {algorithm}")
                aes_key = cipher.decrypt(k)
                aes_cipher = AESCipher(key=AESCipher.get_key_bytes(aes_key))
                return aes_cipher.decrypt(v.decode('utf-8'))
        except Exception as e:
            raise ValueError(f"Decryption failed: {str(e)}")